@@ -18,7 +18,7 @@ import time
 from electrum.crypto import sha256d, EncodeAES_base64, EncodeAES_bytes, DecodeAES_bytes, hmac_oneshot
 from electrum.bitcoin import (TYPE_ADDRESS, push_script, var_int, public_key_to_p2pkh,
                               is_address)
-from electrum.bip32 import serialize_xpub, deserialize_xpub
+from electrum.bip32 import BIP32Node
 from electrum import ecc
 from electrum.ecc import msg_magic
 from electrum.wallet import Standard_Wallet
@@ -27,9 +27,14 @@ from electrum.transaction import Transaction
 from electrum.i18n import _
 from electrum.keystore import Hardware_KeyStore
 from ..hw_wallet import HW_PluginBase
-from electrum.util import print_error, to_string, UserCancelled, UserFacingException
+from electrum.util import to_string, UserCancelled, UserFacingException
 from electrum.base_wizard import ScriptTypeNotSupported, HWD_SETUP_NEW_WALLET
 from electrum.network import Network
+from electrum.logging import get_logger
+
+
+_logger = get_logger(__name__)
+
 
 try:
     import hid
@@ -118,8 +123,8 @@ class DigitalBitbox_Client():
             # only ever returns the mainnet standard type, but it is agnostic
             # to the type when signing.
             if xtype != 'standard' or constants.net.TESTNET:
-                _, depth, fingerprint, child_number, c, cK = deserialize_xpub(xpub, net=constants.BitcoinMainnet)
-                xpub = serialize_xpub(xtype, c, cK, depth, fingerprint, child_number)
+                node = BIP32Node.from_xkey(xpub, net=constants.BitcoinMainnet)
+                xpub = node._replace(xtype=xtype).to_xpub()
             return xpub
         else:
             raise Exception('no reply')
@@ -406,7 +411,7 @@ class DigitalBitbox_Client():
             r = to_string(r, 'utf8')
             reply = json.loads(r)
         except Exception as e:
-            print_error('Exception caught ' + repr(e))
+            _logger.info(f'Exception caught {repr(e)}')
         return reply
 
 
@@ -431,7 +436,7 @@ class DigitalBitbox_Client():
             if 'error' in reply:
                 self.password = None
         except Exception as e:
-            print_error('Exception caught ' + repr(e))
+            _logger.info(f'Exception caught {repr(e)}')
         return reply
 
 
@@ -679,7 +684,7 @@ class DigitalBitbox_KeyStore(Hardware_KeyStore):
         except BaseException as e:
             self.give_error(e, True)
         else:
-            print_error("Transaction is_complete", tx.is_complete())
+            _logger.info("Transaction is_complete {tx.is_complete()}")
             tx.raw = tx.serialize()
 
 
@@ -746,7 +751,7 @@ class DigitalBitboxPlugin(HW_PluginBase):
         )
         try:
             text = Network.send_http_on_proxy('post', url, body=args.encode('ascii'), headers={'content-type': 'application/x-www-form-urlencoded'})
-            print_error('digitalbitbox reply from server', text)
+            _logger.info(f'digitalbitbox reply from server {text}')
         except Exception as e:
             self.handler.show_error(repr(e)) # repr because str(Exception()) == ''
 
